# Copyright (c) 2003-2004 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

__author__ = "Jean-Marc Orliaguet <jmo@ita.chalmers.se>"

"""
  Base Style
"""

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from OFS.PropertyManager import PropertyManager
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.PortalContent import PortalContent

from cpsskins_utils import rebuild_properties, callAction, \
                           getFreeId, getFreeTitle, canonizeStyleTitle, \
                           verifyThemePerms, canonizeId

def addBaseStyle(dispatcher, id, REQUEST=None):
    """Add a Base Style."""
    ob = BaseStyle(id)
    container = dispatcher.Destination()
    container._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


factory_type_information = (
    {'id': 'Base Style',
     'description': ('A Base Style is the most basic style.'),
     'meta_type': 'Base Style',
     'icon': 'style_icon.gif',
     'product': 'CPSSkins',
     'factory': 'addBaseStyle',
     'immediate_view': 'cpsskins_edit_form',
     'filter_content_types': 0,
     'actions': (
         {'id': 'edit',
          'name': 'Edit',
          'action': 'cpsskins_edit_form',
          'permissions': ('Manage Themes',)
         },
         {'id': 'delete',
          'name': 'Delete',
          'action': 'cpsskins_object_delete',
          'visible': 0,
          'permissions': ('Manage Themes',)
         },
         {'id': 'isstyle',
          'name': 'isstyle',
          'action': 'isstyle',
          'visible': 0,
          'permissions': ()},
     ),
    },
)

class BaseStyle(PortalContent, PropertyManager):
    """
    Base class for styles
    """

    meta_type = None
    portal_type = None
    isstyle = 1

    manage_options = ( PropertyManager.manage_options      # Properties
                    + (
                       {'label': 'Preview',
                        'action': 'manage_stylePreview'}, 
                      )
                     )

    security = ClassSecurityInfo()
    security.declareProtected(View, 'manage_stylePreview')
    manage_stylePreview = DTMLFile('zmi/manage_stylePreview', globals())

    _actions = factory_type_information[0]['actions']

    _properties = (
        {'id': 'title', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Title'},
        )

    def __init__(self, id, 
                       title= 'Style', 
                       **kw):
        self.id = id
        self.title = title

                                
    security.declareProtected(View,'isPortalStyle')
    def isPortalStyle(self):
        """ is style """
           
        return 1

    security.declareProtected(View,'isPortalContent')
    def isPortalContent(self):
        """ is portal content """
                  
        return 0  

    security.declarePublic('getIconRelative')
    def getIconRelative(self):
        """
        Gets the Icon path, relative to the portal.
        This is needed to catalog correctly in the presence of VHM.
        """
        return self.getIcon(1)


    security.declarePublic('getTitle')
    def getTitle(self):
        """
        Gets the style's title
        """

        return getattr(self, 'title', None)


    security.declareProtected('Manage Themes', 'rebuild')
    def rebuild(self):
        """                  
        Rebuild this style
        """              

        canonizeId(self)
        rebuild_properties(self)
        verifyThemePerms(self)


    security.declareProtected('Manage Themes', 'edit_form')
    def edit_form(self, **kw):
        """
        Call the edit action.
        """
        return callAction(self, 'edit', **kw)


    security.declareProtected('Manage Themes', 'edit')
    def edit(self, **kw):
        """
        Default edit method, changes the properties.
        """

        tmtool = self.portal_themes
        title = kw.get('title', '')
        current_title = self.getTitle()
        if title == '':
            kw['title'] = current_title
        else:
            title = canonizeStyleTitle(title)
            kw['title'] = title
            if title != current_title:
                theme_container = tmtool.getPortalThemeRoot(self)
                styles_dir = theme_container.getStylesFolder()

                new_title = getFreeTitle(styles_dir, title)
                kw['title'] = new_title

                self.findParents(newtitle=new_title)
        self.manage_changeProperties(**kw)


    security.declareProtected(View, 'findParents')
    def findParents(self, newtitle=None):
        """
        Find the style's parents
        If the 'newtitle' parameter is passed, the style's title will be updated in every object 
        that uses the style.
        """
        style_title = self.getTitle()
        meta_type = self.meta_type
       
        # XXX needs some refactoring 
        tmtool = self.portal_themes
        theme_container = tmtool.getPortalThemeRoot(self)
        parents = []

        for obj in theme_container.objectValues('Page Block'):
               for pm in obj.propertyMap():
                  if pm.get('meta_type', None) == meta_type:
                     style_type = pm.get('id')
                     style = getattr(obj, style_type, None)
                     if style == style_title:
                        parents.append(obj)
                        if newtitle is not None:
                           prop_dict = {}
                           prop_dict[style_type] = newtitle
                           obj.manage_changeProperties(prop_dict)

               for obj2 in obj.objectValues():
                  if hasattr(obj2, 'propertyMap'):
                     for pm in obj2.propertyMap():
                        if pm.get('meta_type', None) == meta_type:
                            style_type = pm.get('id')
                            style = getattr(obj2, style_type, None)
                            if style == style_title:
                               parents.append(obj2)
                               if newtitle is not None:
                                   prop_dict = {}
                                   prop_dict[style_type] = newtitle
                                   obj2.manage_changeProperties(prop_dict)

        if hasattr(theme_container, 'propertyMap'):
           for pm in theme_container.propertyMap():
               if pm.get('meta_type', None) == meta_type:
                  style_type = pm.get('id')
                  style = getattr(theme_container, style_type, None)
                  if style == style_title:
                     parents.append(theme_container)
                     if newtitle is not None:
                        prop_dict = {}
                        prop_dict[style_type] = newtitle
                        theme_container.manage_changeProperties(prop_dict)

        return parents



    security.declareProtected(View, 'isOrphan')
    def isOrphan(self):
        """
        Tells whether the style is an orphan 
        """

        parents = self.findParents()
        if len(parents) == 0:
       	    return 1


    security.declareProtected('Manage Themes', 'duplicate')
    def duplicate(self):
        """
        Duplicate a Style
        """
        
        container = self.aq_parent
        newid = getFreeId(container)
        container.manage_clone(self, newid)
        newobj = getattr(container, newid, None)
        newtitle = getFreeTitle(container, self.getTitle())
        newobj.manage_changeProperties({'title':newtitle})
        newobj.manage_permission('View',(),acquire=1)
        newobj.manage_permission('Modify portal content',(),acquire=1)
        return newobj


    security.declareProtected(View, 'can_delete')
    def can_delete(self):
        """
        Can the style be deleted ?
        """

        return 1


    security.declarePublic('getInfo')
    def getInfo(self):

        infoblock = {}
        actions_list = ['delete', ]

        ti = self.getTypeInfo()
        for actionid in actions_list:  
            actioninfo = {}
            if actionid == 'delete':
                actioninfo['can_delete'] = self.can_delete()
            try:
                action = ti.getActionById(actionid)
                actioninfo['url'] = self.absolute_url() + '/' + \
                                    self.restrictedTraverse(action).getId()
            except:
                continue 
            infoblock[actionid] = actioninfo
        return infoblock 

    security.declareProtected('Manage Themes', 'preview')
    def preview(self, *args, **kw):
        """
        Renders a preview of the style 
        """

        return callAction(self, 'preview', **kw)


    security.declareProtected('Manage Themes', 'list')
    def list(self, *args, **kw):
        """
        List available styles
        """

        return callAction(self, 'list', **kw)


    security.declareProtected(View, 'render')
    def render(self, *args, **kw):
        """
        Renders the style.
        """

        fail = 0
        if hasattr(self.aq_explicit, 'render_action'):
            actionid = self.render_action
        else:
            ti = self.getTypeInfo() 
            try: 
                actionid = ti.getActionById('render')
            except:
                fail = 1

        if not fail:
            meth = getattr(self, actionid, None)
            if meth is not None:
                if getattr(aq_base(meth), 'isDocTemp', 0):
                    rendered = apply(meth, (self, self.REQUEST), kw)
                else:
                    rendered = apply(meth, (), kw)
            else:
                fail = 1
                       
        if not fail:
            return rendered  


InitializeClass(BaseStyle)
